@@ -219,7 +219,7 @@ def setup(
 def up(
     ws_filter: Optional[str] = typer.Argument(
         None,
-        help="Filter the resources to deploy. Format - ENV:NAME:TYPE:CONFIG",
+        help="Filter the resources to deploy. Format - ENV:APP:NAME:TYPE:CONFIG",
         metavar="[filter]",
     ),
     env_filter: Optional[str] = typer.Option(
@@ -250,12 +250,20 @@ def up(
         metavar="",
         help="Filter using resource type",
     ),
+    app_filter: Optional[str] = typer.Option(
+        None, "-a", "--app", metavar="", help="Filter using app name"
+    ),
     dry_run: bool = typer.Option(
         False,
         "-dr",
         "--dry-run",
         help="Print which resources will be deployed and exit.",
     ),
+    auto_confirm: bool = typer.Option(
+        False,
+        "-y",
+        help="Skip the confirmation before deploying resources.",
+    ),
     print_debug_log: bool = typer.Option(
         False,
         "-d",
@@ -267,12 +275,13 @@ def up(
     \b
     Deploys the active workspace.
     Filters can be used to limit deployment by
-        - env (dev, stg, prd)
-        - config (docker, aws, k8s)
-        - Resource Name
-        - Resource Type
+        - Env (dev, stg, prd)
+        - App name
+        - Resource name
+        - Resource type
+        - Config type (docker, aws, k8s)
     \b
-    Filters can be provided as a single argument as - ENV:NAME:TYPE:CONFIG
+    Filters can be provided as a single argument as - ENV:APP:NAME:TYPE:CONFIG
     or using multiple options.
     Examples:
     \b
@@ -280,7 +289,7 @@ def up(
     $ `phi ws up dev`       -> Deploy all dev resources
     $ `phi ws up prd`       -> Deploy all prd resources
     $ `phi ws up prd:::aws` -> Deploy all prd aws resources
-    $ `phi ws up prd:s3`    -> Deploy prd resources with name s3
+    $ `phi ws up prd::s3`   -> Deploy prd resources with name s3
     """
     from phiterm.conf.phi_conf import PhiConf, PhiWsData
     from phiterm.workspace.ws_operator import deploy_workspace
@@ -307,6 +316,7 @@ def up(
     target_config: Optional[WorkspaceConfigType] = None
     target_name: Optional[str] = None
     target_type: Optional[str] = None
+    target_app: Optional[str] = None
 
     # derive env/name/type/config from ws_filter
     if ws_filter is not None:
@@ -315,26 +325,46 @@ def up(
                 f"Invalid workspace filter. Expected: str, Received: {type(ws_filter)}"
             )
         filters = ws_filter.split(":")
-        # logger.debug(f"filters: {filters}")
+        # logger.debug(f"Filters: {filters}")
         num_filters = len(filters)
         if num_filters >= 1:
             target_env_str = filters[0]
         if num_filters >= 2:
-            target_name = filters[1]
+            if filters[1] != "":
+                target_app = filters[1]
         if num_filters >= 3:
-            target_type = filters[2]
+            if filters[2] != "":
+                target_name = filters[2]
         if num_filters >= 4:
-            target_config_str = filters[3]
+            if filters[3] != "":
+                target_type = filters[3]
+        if num_filters >= 5:
+            if filters[4] != "":
+                target_config_str = filters[4]
 
-    # derive env/config/name/type from command options
-    if target_env_str is None:
+    # derive env/app/name/type/config from command options
+    if (
+        target_env_str is None
+        and env_filter is not None
+        and isinstance(env_filter, str)
+    ):
         target_env_str = env_filter
-    if target_config_str is None:
+    if (
+        target_config_str is None
+        and config_filter is not None
+        and isinstance(config_filter, str)
+    ):
         target_config_str = config_filter
-    if target_name is None:
+    if target_name is None and name_filter is not None and isinstance(name_filter, str):
         target_name = name_filter
-    if target_type is None:
+    if target_type is None and type_filter is not None and isinstance(type_filter, str):
         target_type = type_filter
+    if (
+        target_app is not None
+        and app_filter is not None
+        and isinstance(app_filter, str)
+    ):
+        target_app = app_filter
 
     # derive env/config/name/type from defaults
     if target_env_str is None:
@@ -371,10 +401,11 @@ def up(
         )
 
     logger.debug("Deploying workspace")
-    logger.debug(f"\ttarget_env  : {target_env}")
+    logger.debug(f"\ttarget_env   : {target_env}")
     logger.debug(f"\ttarget_config: {target_config}")
-    logger.debug(f"\ttarget_name : {target_name}")
-    logger.debug(f"\ttarget_type : {target_type}")
+    logger.debug(f"\ttarget_name  : {target_name}")
+    logger.debug(f"\ttarget_type  : {target_type}")
+    logger.debug(f"\ttarget_app   : {target_app}")
     print_heading("Deploying workspace: {}\n".format(active_ws_data.ws_name))
     deploy_workspace(
         ws_data=active_ws_data,
@@ -382,7 +413,9 @@ def up(
         target_config=target_config,
         target_name=target_name,
         target_type=target_type,
+        target_app=target_app,
         dry_run=dry_run,
+        auto_confirm=auto_confirm,
     )
 
 
@@ -390,7 +423,7 @@ def up(
 def down(
     ws_filter: Optional[str] = typer.Argument(
         None,
-        help="Filter the resources to shut down. Format - ENV:NAME:TYPE:CONFIG",
+        help="Filter the resources to shut down. Format - ENV:APP:NAME:TYPE:CONFIG",
         metavar="[filter]",
     ),
     env_filter: str = typer.Option(
@@ -421,6 +454,9 @@ def down(
         metavar="",
         help="Filter using resource type",
     ),
+    app_filter: Optional[str] = typer.Option(
+        None, "-a", "--app", metavar="", help="Filter using app name"
+    ),
     dry_run: bool = typer.Option(
         False,
         "-dr",
@@ -467,6 +503,7 @@ def down(
     target_config: Optional[WorkspaceConfigType] = None
     target_name: Optional[str] = None
     target_type: Optional[str] = None
+    target_app: Optional[str] = None
 
     # derive env/config/name/type from ws_filter
     if ws_filter is not None:
@@ -475,32 +512,58 @@ def down(
                 f"Invalid workspace filter. Expected: str, Received: {type(ws_filter)}"
             )
         filters = ws_filter.split(":")
-        # logger.debug(f"filters: {filters}")
+        # logger.debug(f"Filters: {filters}")
         num_filters = len(filters)
         if num_filters >= 1:
             target_env_str = filters[0]
         if num_filters >= 2:
-            target_name = filters[1]
+            if filters[1] != "":
+                target_app = filters[1]
         if num_filters >= 3:
-            target_type = filters[2]
+            if filters[2] != "":
+                target_name = filters[2]
         if num_filters >= 4:
-            target_config_str = filters[3]
+            if filters[3] != "":
+                target_type = filters[3]
+        if num_filters >= 5:
+            if filters[4] != "":
+                target_config_str = filters[4]
 
-    # derive env/config/name/type from command options
-    if target_env_str is None:
+    # derive env/app/name/type/config from command options
+    if (
+        target_env_str is None
+        and env_filter is not None
+        and isinstance(env_filter, str)
+    ):
         target_env_str = env_filter
-    if target_config_str is None:
+    if (
+        target_config_str is None
+        and config_filter is not None
+        and isinstance(config_filter, str)
+    ):
         target_config_str = config_filter
-    if target_name is None:
+    if target_name is None and name_filter is not None and isinstance(name_filter, str):
         target_name = name_filter
-    if target_type is None:
+    if target_type is None and type_filter is not None and isinstance(type_filter, str):
         target_type = type_filter
+    if (
+        target_app is not None
+        and app_filter is not None
+        and isinstance(app_filter, str)
+    ):
+        target_app = app_filter
 
     # derive env/config/name/type from defaults
     if target_env_str is None:
-        target_env_str = active_ws_data.ws_config.default_env
+        target_env_str = (
+            active_ws_data.ws_config.default_env if active_ws_data.ws_config else None
+        )
     if target_config_str is None:
-        target_config_str = active_ws_data.ws_config.default_config
+        target_config_str = (
+            active_ws_data.ws_config.default_config
+            if active_ws_data.ws_config
+            else None
+        )
 
     if target_env_str is not None:
         if target_env_str.lower() not in WorkspaceEnv.values_list():
@@ -525,10 +588,11 @@ def down(
         )
 
     logger.debug("Shutting down workspace")
-    logger.debug(f"\ttarget_env: {target_env}")
+    logger.debug(f"\ttarget_env   : {target_env}")
     logger.debug(f"\ttarget_config: {target_config}")
-    logger.debug(f"\ttarget_name: {target_name}")
-    logger.debug(f"\ttarget_type: {target_type}")
+    logger.debug(f"\ttarget_name  : {target_name}")
+    logger.debug(f"\ttarget_type  : {target_type}")
+    logger.debug(f"\ttarget_app   : {target_app}")
     print_heading("Shutdown workspace: {}\n".format(active_ws_data.ws_name))
     shutdown_workspace(
         ws_data=active_ws_data,
@@ -536,6 +600,7 @@ def down(
         target_config=target_config,
         target_name=target_name,
         target_type=target_type,
+        target_app=target_app,
         dry_run=dry_run,
     )
 
@@ -544,7 +609,7 @@ def down(
 def patch(
     ws_filter: Optional[str] = typer.Argument(
         None,
-        help="Filter the resources to patch. Format - ENV:NAME:TYPE:CONFIG",
+        help="Filter the resources to patch. Format - ENV:APP:NAME:TYPE:CONFIG",
         metavar="[filter]",
     ),
     env_filter: str = typer.Option(
@@ -575,6 +640,9 @@ def patch(
         metavar="",
         help="Filter using resource type",
     ),
+    app_filter: Optional[str] = typer.Option(
+        None, "-a", "--app", metavar="", help="Filter using app name"
+    ),
     dry_run: bool = typer.Option(
         False,
         "-dr",
@@ -592,12 +660,13 @@ def patch(
     \b
     Patch the active workspace.
     Filters can be used to limit patching by
-        - env (dev, stg, prd)
-        - config (docker, aws, k8s)
-        - Resource Name
-        - Resource Type
+        - Env (dev, stg, prd)
+        - App name
+        - Resource name
+        - Resource type
+        - Config type (docker, aws, k8s)
     \b
-    Filters can be provided as a single argument as - ENV:NAME:TYPE:CONFIG
+    Filters can be provided as a single argument as - ENV:APP:NAME:TYPE:CONFIG
     or using multiple options.
     Examples:
     \b
@@ -605,7 +674,7 @@ def patch(
     $ `phi ws patch dev`       -> Patch all dev resources
     $ `phi ws patch prd`       -> Patch all prd resources
     $ `phi ws patch prd:::aws` -> Patch all prd aws resources
-    $ `phi ws patch prd:s3`    -> Patch prd resources with name s3
+    $ `phi ws patch prd::s3`   -> Patch prd resources with name s3
     """
     from phiterm.conf.phi_conf import PhiConf, PhiWsData
     from phiterm.workspace.ws_operator import patch_workspace
@@ -632,6 +701,7 @@ def patch(
     target_config: Optional[WorkspaceConfigType] = None
     target_name: Optional[str] = None
     target_type: Optional[str] = None
+    target_app: Optional[str] = None
 
     # derive env/name/type/config from ws_filter
     if ws_filter is not None:
@@ -640,26 +710,46 @@ def patch(
                 f"Invalid workspace filter. Expected: str, Received: {type(ws_filter)}"
             )
         filters = ws_filter.split(":")
-        # logger.debug(f"filters: {filters}")
+        # logger.debug(f"Filters: {filters}")
         num_filters = len(filters)
         if num_filters >= 1:
             target_env_str = filters[0]
         if num_filters >= 2:
-            target_name = filters[1]
+            if filters[1] != "":
+                target_app = filters[1]
         if num_filters >= 3:
-            target_type = filters[2]
+            if filters[2] != "":
+                target_name = filters[2]
         if num_filters >= 4:
-            target_config_str = filters[3]
+            if filters[3] != "":
+                target_type = filters[3]
+        if num_filters >= 5:
+            if filters[4] != "":
+                target_config_str = filters[4]
 
-    # derive env/config/name/type from command options
-    if target_env_str is None:
+    # derive env/app/name/type/config from command options
+    if (
+        target_env_str is None
+        and env_filter is not None
+        and isinstance(env_filter, str)
+    ):
         target_env_str = env_filter
-    if target_config_str is None:
+    if (
+        target_config_str is None
+        and config_filter is not None
+        and isinstance(config_filter, str)
+    ):
         target_config_str = config_filter
-    if target_name is None:
+    if target_name is None and name_filter is not None and isinstance(name_filter, str):
         target_name = name_filter
-    if target_type is None:
+    if target_type is None and type_filter is not None and isinstance(type_filter, str):
         target_type = type_filter
+    if (
+        target_app is not None
+        and app_filter is not None
+        and isinstance(app_filter, str)
+    ):
+        target_app = app_filter
 
     # derive env/config/name/type from defaults
     if target_env_str is None:
@@ -690,10 +780,11 @@ def patch(
         )
 
     logger.debug("Patching workspace")
-    logger.debug(f"\ttarget_env: {target_env}")
+    logger.debug(f"\ttarget_env   : {target_env}")
     logger.debug(f"\ttarget_config: {target_config}")
-    logger.debug(f"\ttarget_name: {target_name}")
-    logger.debug(f"\ttarget_type: {target_type}")
+    logger.debug(f"\ttarget_name  : {target_name}")
+    logger.debug(f"\ttarget_type  : {target_type}")
+    logger.debug(f"\ttarget_app   : {target_app}")
     print_heading("Patching workspace: {}\n".format(active_ws_data.ws_name))
     patch_workspace(
         ws_data=active_ws_data,
@@ -701,6 +792,7 @@ def patch(
         target_config=target_config,
         target_name=target_name,
         target_type=target_type,
+        target_app=target_app,
         dry_run=dry_run,
     )
 
@@ -709,7 +801,7 @@ def patch(
 def restart(
     ws_filter: Optional[str] = typer.Argument(
         None,
-        help="Filter the resources to restart. Format - ENV:NAME:TYPE:CONFIG",
+        help="Filter the resources to restart. Format - ENV:APP:NAME:TYPE:CONFIG",
         metavar="[filter]",
     ),
     env_filter: str = typer.Option(
@@ -740,6 +832,9 @@ def restart(
         metavar="",
         help="Filter using resource type",
     ),
+    app_filter: Optional[str] = typer.Option(
+        None, "-a", "--app", metavar="", help="Filter using app name"
+    ),
     print_debug_log: bool = typer.Option(
         False,
         "-d",
@@ -763,6 +858,7 @@ def restart(
         config_filter=config_filter,
         name_filter=name_filter,
         type_filter=type_filter,
+        app_filter=app_filter,
         dry_run=False,
         print_debug_log=print_debug_log,
     )
@@ -774,6 +870,7 @@ def restart(
         config_filter=config_filter,
         name_filter=name_filter,
         type_filter=type_filter,
+        app_filter=app_filter,
         dry_run=False,
         print_debug_log=print_debug_log,
     )
