@@ -492,16 +492,17 @@ class PhiConf:
         return None
 
     def print_to_cli(self):
-        print_heading("Phidata Config\n")
         if self.user:
             print_info(f"* User: {self.user.email}")
         if self.active_ws_name:
-            print_info(f"* Active workspace: {self.active_ws_name}")
+            print_heading(f"\nActive workspace: {self.active_ws_name}\n")
         else:
             print_info("* No active workspace, run `phi ws init` to create one")
         if len(self._ws_data_map) > 0:
-            print_subheading("\nAvailable workspaces")
+            print_heading("Available workspaces:\n")
+            c = 1
             for k, v in self._ws_data_map.items():
-                print_info(f"Name: {v.ws_name}")
-                print_info(f"Dir: {v.ws_root_path}\n")
-                print_horizontal_line()
+                print_info(f"  {c}. Name: {v.ws_name}")
+                print_info(f"     Dir: {v.ws_root_path}")
+                print_info(f"  -*-\n")
+                c += 1
