@@ -31,38 +31,46 @@ def deploy_k8s_config(
     config: K8sConfig,
     name_filter: Optional[str] = None,
     type_filter: Optional[str] = None,
+    app_filter: Optional[str] = None,
     dry_run: Optional[bool] = False,
+    auto_confirm: Optional[bool] = False,
 ) -> bool:
 
     # Step 1: Get the K8sManager
     k8s_manager: K8sManager = config.get_k8s_manager()
     if k8s_manager is None:
         raise K8sConfigException("K8sManager unavailable")
+    print_heading(f"--**-- K8s env: {config.env}\n")
 
-    # Step 3: If dry_run, print the resources and return True
+    # Step 2: If dry_run, print the resources and return True
     if dry_run:
         k8s_manager.create_resources_dry_run(
-            name_filter=name_filter, type_filter=type_filter
+            name_filter=name_filter,
+            type_filter=type_filter,
+            app_filter=app_filter,
+            auto_confirm=auto_confirm,
         )
         return True
 
     # Step 3: Create resources
-    env = config.env
-    print_subheading(
-        "Deploying k8s config{}\n".format(f" for env: {env}" if env is not None else "")
-    )
     try:
         success: bool = k8s_manager.create_resources(
-            name_filter=name_filter, type_filter=type_filter
+            name_filter=name_filter,
+            type_filter=type_filter,
+            app_filter=app_filter,
+            auto_confirm=auto_confirm,
         )
         if not success:
             return False
-    except Exception:
-        raise
+    except Exception as e:
+        logger.error(e)
+        return False
 
     # Step 4: Validate resources are created
     resource_creation_valid: bool = k8s_manager.validate_resources_are_created(
-        name_filter=name_filter, type_filter=type_filter
+        name_filter=name_filter,
+        type_filter=type_filter,
+        app_filter=app_filter,
     )
     if not resource_creation_valid:
         logger.error("K8sResource creation could not be validated")
@@ -76,46 +84,52 @@ def shutdown_k8s_config(
     config: K8sConfig,
     name_filter: Optional[str] = None,
     type_filter: Optional[str] = None,
+    app_filter: Optional[str] = None,
     dry_run: Optional[bool] = False,
+    auto_confirm: Optional[bool] = False,
 ) -> bool:
 
     # Step 1: Get the K8sManager
     k8s_manager: K8sManager = config.get_k8s_manager()
     if k8s_manager is None:
         raise K8sConfigException("K8sManager unavailable")
+    print_heading(f"--**-- K8s env: {config.env}\n")
 
     # Step 2: If dry_run, print the resources and return True
     if dry_run:
         k8s_manager.delete_resources_dry_run(
-            name_filter=name_filter, type_filter=type_filter
+            name_filter=name_filter,
+            type_filter=type_filter,
+            app_filter=app_filter,
+            auto_confirm=auto_confirm,
         )
         return True
 
     # Step 3: Delete resources
-    env = config.env
-    print_subheading(
-        "Shutting down k8s config{}\n".format(
-            f" for env: {env}" if env is not None else ""
-        )
-    )
     try:
         success: bool = k8s_manager.delete_resources(
-            name_filter=name_filter, type_filter=type_filter
+            name_filter=name_filter,
+            type_filter=type_filter,
+            app_filter=app_filter,
+            auto_confirm=auto_confirm,
         )
         if not success:
             return False
-    except Exception:
-        raise
+    except Exception as e:
+        logger.error(e)
+        return False
 
     # Step 4: Validate resources are deleted
     resources_deletion_valid: bool = k8s_manager.validate_resources_are_deleted(
-        name_filter=name_filter, type_filter=type_filter
+        name_filter=name_filter,
+        type_filter=type_filter,
+        app_filter=app_filter,
     )
     if not resources_deletion_valid:
         logger.error("K8sResource deletion could not be validated")
         return False
 
-    print_info("K8s config shutdown")
+    print_info("K8s config shut down")
     return True
 
 
@@ -123,38 +137,46 @@ def patch_k8s_config(
     config: K8sConfig,
     name_filter: Optional[str] = None,
     type_filter: Optional[str] = None,
+    app_filter: Optional[str] = None,
     dry_run: Optional[bool] = False,
+    auto_confirm: Optional[bool] = False,
 ) -> bool:
 
     # Step 1: Get the K8sManager
     k8s_manager: K8sManager = config.get_k8s_manager()
     if k8s_manager is None:
         raise K8sConfigException("K8sManager unavailable")
+    print_heading(f"--**-- K8s env: {config.env}\n")
 
     # Step 2: If dry_run, print the resources and return True
     if dry_run:
         k8s_manager.patch_resources_dry_run(
-            name_filter=name_filter, type_filter=type_filter
+            name_filter=name_filter,
+            type_filter=type_filter,
+            app_filter=app_filter,
+            auto_confirm=auto_confirm,
         )
         return True
 
     # Step 3: Patch resources
-    env = config.env
-    print_subheading(
-        "Patching k8s config{}\n".format(f" for env: {env}" if env is not None else "")
-    )
     try:
         success: bool = k8s_manager.patch_resources(
-            name_filter=name_filter, type_filter=type_filter
+            name_filter=name_filter,
+            type_filter=type_filter,
+            app_filter=app_filter,
+            auto_confirm=auto_confirm,
         )
         if not success:
             return False
-    except Exception:
-        raise
+    except Exception as e:
+        logger.error(e)
+        return False
 
     # Step 4: Validate resources are patched
     resources_patch_valid: bool = k8s_manager.validate_resources_are_patched(
-        name_filter=name_filter, type_filter=type_filter
+        name_filter=name_filter,
+        type_filter=type_filter,
+        app_filter=app_filter,
     )
     if not resources_patch_valid:
         logger.error("K8sResource patch could not be validated")
