@@ -1,5 +1,6 @@
 from pathlib import Path
 from typing import List, Optional, cast
+from typing_extensions import Literal
 
 import git
 import typer
@@ -532,51 +533,53 @@ def setup_workspace(ws_root_path: Path) -> bool:
     ######################################################
 
 
-def deploy_workspace(
+def filter_and_prep_configs(
     ws_data: PhiWsData,
     target_env: Optional[WorkspaceEnv] = None,
     target_config: Optional[WorkspaceConfigType] = None,
-    target_name: Optional[str] = None,
-    target_type: Optional[str] = None,
-    dry_run: Optional[bool] = False,
-) -> None:
-    """Deploy a Phidata Workspace. This is called from `phi ws up`"""
+    order: Literal["create", "delete"] = "create",
+) -> List[InfraConfig]:
 
-    if ws_data is None or ws_data.ws_config is None:
-        logger.error("WorkspaceConfig invalid")
-        return
     ws_config: WorkspaceConfig = ws_data.ws_config
 
-    # Add configs in the order they should be deployed
-    configs_to_deploy_infra_filter: List[InfraConfig] = []
+    # Step 1. Filter configs
+    # 1.1: Filter by config type: docker/k8s/aws
+    filtered_configs: List[InfraConfig] = []
     if target_config is None:
         if ws_config.docker is not None:
-            configs_to_deploy_infra_filter.extend(ws_config.docker)
-        if ws_config.aws is not None:
-            configs_to_deploy_infra_filter.extend(ws_config.aws)
-        if ws_config.k8s is not None:
-            configs_to_deploy_infra_filter.extend(ws_config.k8s)
+            filtered_configs.extend(ws_config.docker)
+        if order == "delete":
+            if ws_config.k8s is not None:
+                filtered_configs.extend(ws_config.k8s)
+            if ws_config.aws is not None:
+                filtered_configs.extend(ws_config.aws)
+        else:
+            if ws_config.aws is not None:
+                filtered_configs.extend(ws_config.aws)
+            if ws_config.k8s is not None:
+                filtered_configs.extend(ws_config.k8s)
     elif target_config == WorkspaceConfigType.docker:
         if ws_config.docker is not None:
-            configs_to_deploy_infra_filter.extend(ws_config.docker)
+            filtered_configs.extend(ws_config.docker)
         else:
             logger.error("No DockerConfig provided")
     elif target_config == WorkspaceConfigType.k8s:
         if ws_config.k8s is not None:
-            configs_to_deploy_infra_filter.extend(ws_config.k8s)
+            filtered_configs.extend(ws_config.k8s)
         else:
             logger.error("No K8sConfig provided")
     elif target_config == WorkspaceConfigType.aws:
         if ws_config.aws is not None:
-            configs_to_deploy_infra_filter.extend(ws_config.aws)
+            filtered_configs.extend(ws_config.aws)
         else:
             logger.error("No AwsConfig provided")
 
-    configs_to_deploy_env_filter: List[InfraConfig] = []
-    if target_env is None:
-        configs_to_deploy_env_filter = configs_to_deploy_infra_filter
+    # 1.2: Filter by env: dev/stg/prd
+    configs_after_env_filter: List[InfraConfig] = []
+    if target_env is None or target_env == WorkspaceEnv.all:
+        configs_after_env_filter = filtered_configs
     else:
-        for config in configs_to_deploy_infra_filter:
+        for config in filtered_configs:
             if config.env is None:
                 continue
             if config.env.lower() not in WorkspaceEnv.values_list():
@@ -585,12 +588,11 @@ def deploy_workspace(
             config_env = WorkspaceEnv.from_str(config.env)
 
             if config_env == target_env:
-                configs_to_deploy_env_filter.append(config)
-    # logger.debug("Deploying configs: {}".format(configs_to_deploy_env_filter))
+                configs_after_env_filter.append(config)
+    # logger.debug("Filtered configs: {}".format(configs_after_env_filter))
 
-    num_configs_to_process = len(configs_to_deploy_env_filter)
-    num_configs_processed = 0
-    for config in configs_to_deploy_env_filter:
+    # Step 2. Prepare configs
+    for config in configs_after_env_filter:
         if not isinstance(config, InfraConfig):
             logger.error(f"{config} is not an instance of InfraConfig")
             continue
@@ -601,18 +603,18 @@ def deploy_workspace(
             logger.debug(
                 f"{config.__class__.__name__} for env {config.__class__.env} disabled"
             )
-            num_configs_processed += 1
             continue
 
         ######################################################################
         # NOTE: VERY IMPORTANT TO GET RIGHT
-        # Update sub-configs data using WorkspaceConfig
+        # Update subconfigs using the WorkspaceConfig
         # 1. Pass down the paths from the WorkspaceConfig
-        #       These paths are used everywhere from Infra to Apps
+        #       These paths are used everywhere from Infra, Apps, Resources, Workflows
         # 2. Pass down local_env, docker_env, k8s_env for building PhidataApps
         # 3. Pass down common cloud configuration. eg: aws_region, aws_profile
         ######################################################################
 
+        # -*- Path parameters
         # The ws_root_path is the ROOT directory for the workspace
         config.workspace_root_path = ws_data.ws_root_path
         config.workspace_config_file_path = ws_data.ws_config_file_path
@@ -623,52 +625,103 @@ def deploy_workspace(
         config.notebooks_dir = ws_config.notebooks_dir
         config.workspace_config_dir = ws_config.workspace_config_dir
 
+        # -*- Environment parameters
+        # only update the param if they are not available.
+        # prefer the configs param if provided
+        if config.local_env is None and ws_config.local_env is not None:
+            config.local_env = ws_config.local_env
+        if config.docker_env is None and ws_config.docker_env is not None:
+            config.docker_env = ws_config.docker_env
+        if config.k8s_env is None and ws_config.k8s_env is not None:
+            config.k8s_env = ws_config.k8s_env
+
+        # -*- AWS parameters
+        # only update the param if they are available.
+        # prefer the configs param if provided
+        if config.aws_region is None and ws_config.aws_region is not None:
+            config.aws_region = ws_config.aws_region
+        if config.aws_profile is None and ws_config.aws_profile is not None:
+            config.aws_profile = ws_config.aws_profile
+        if config.aws_config_file is None and ws_config.aws_config_file is not None:
+            config.aws_config_file = ws_config.aws_config_file
+        if (
+            config.aws_shared_credentials_file is None
+            and ws_config.aws_shared_credentials_file is not None
+        ):
+            config.aws_shared_credentials_file = ws_config.aws_shared_credentials_file
+
+    # Return filtered and prepared configs
+    return configs_after_env_filter
+
+
+def deploy_workspace(
+    ws_data: PhiWsData,
+    target_env: Optional[WorkspaceEnv] = None,
+    target_config: Optional[WorkspaceConfigType] = None,
+    target_name: Optional[str] = None,
+    target_type: Optional[str] = None,
+    target_app: Optional[str] = None,
+    dry_run: Optional[bool] = False,
+    auto_confirm: Optional[bool] = False,
+) -> None:
+    """Deploy a Phidata Workspace. This is called from `phi ws up`"""
+
+    if ws_data is None or ws_data.ws_config is None:
+        logger.error("WorkspaceConfig invalid")
+        return
+
+    configs_to_deploy: List[InfraConfig] = filter_and_prep_configs(
+        ws_data=ws_data,
+        target_env=target_env,
+        target_config=target_config,
+        order="create",
+    )
+
+    num_configs_to_deploy = len(configs_to_deploy)
+    num_configs_deployed = 0
+    for config in configs_to_deploy:
         logger.debug(f"Deploying {config.__class__.__name__}")
         if isinstance(config, DockerConfig):
             from phiterm.docker.docker_operator import deploy_docker_config
 
-            # Pass down docker_env from WorkspaceConfig
-            config.docker_env = ws_config.docker_env
             deploy_docker_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_deployed += 1
         if isinstance(config, K8sConfig):
             from phiterm.k8s.k8s_operator import deploy_k8s_config
 
-            # Pass down k8s_env from WorkspaceConfig
-            config.k8s_env = ws_config.k8s_env
             deploy_k8s_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_deployed += 1
         if isinstance(config, AwsConfig):
             from phiterm.aws.aws_operator import deploy_aws_config
 
-            # Pass down common aws configuration from WorkspaceConfig
-            # logger.debug("Updating AwsConfig from WorkspaceConfig")
-            config.aws_region = ws_config.aws_region
-            config.aws_profile = ws_config.aws_profile
-            config.aws_config_file = ws_config.aws_config_file
-            config.aws_shared_credentials_file = ws_config.aws_shared_credentials_file
             deploy_aws_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_deployed += 1
+        # white space between runs
+        print_info("")
 
-    print_info(
-        f"\n# Configs processed: {num_configs_processed}/{num_configs_to_process}"
-    )
-    if num_configs_to_process == num_configs_processed:
+    print_info(f"# Configs deployed: {num_configs_deployed}/{num_configs_to_deploy}\n")
+    if num_configs_to_deploy == num_configs_deployed:
         if not dry_run:
             print_subheading("Workspace deploy success")
     else:
@@ -681,7 +734,9 @@ def shutdown_workspace(
     target_config: Optional[WorkspaceConfigType] = None,
     target_name: Optional[str] = None,
     target_type: Optional[str] = None,
+    target_app: Optional[str] = None,
     dry_run: Optional[bool] = False,
+    auto_confirm: Optional[bool] = False,
 ) -> None:
     """Shutdown the Phidata Workspace. This is called from `phi ws down`"""
 
@@ -689,128 +744,60 @@ def shutdown_workspace(
         logger.error("WorkspaceConfig invalid")
         return
 
-    ws_config: WorkspaceConfig = ws_data.ws_config
-
-    # Add configs in the order they should be shut down
-    configs_to_shutdown_infra_filter: List[InfraConfig] = []
-    if target_config is None:
-        if ws_config.docker is not None:
-            configs_to_shutdown_infra_filter.extend(ws_config.docker)
-        if ws_config.k8s is not None:
-            configs_to_shutdown_infra_filter.extend(ws_config.k8s)
-        if ws_config.aws is not None:
-            configs_to_shutdown_infra_filter.extend(ws_config.aws)
-    elif target_config == WorkspaceConfigType.docker:
-        if ws_config.docker is not None:
-            configs_to_shutdown_infra_filter.extend(ws_config.docker)
-        else:
-            logger.error("No DockerConfig provided")
-    elif target_config == WorkspaceConfigType.k8s:
-        if ws_config.k8s is not None:
-            configs_to_shutdown_infra_filter.extend(ws_config.k8s)
-        else:
-            logger.error("No K8sConfig provided")
-    elif target_config == WorkspaceConfigType.aws:
-        if ws_config.aws is not None:
-            configs_to_shutdown_infra_filter.extend(ws_config.aws)
-        else:
-            logger.error("No AwsConfig provided")
-
-    configs_to_shutdown_env_filter: List[InfraConfig] = []
-    if target_env is None:
-        configs_to_shutdown_env_filter = configs_to_shutdown_infra_filter
-    else:
-        for config in configs_to_shutdown_infra_filter:
-            if config.env is None:
-                continue
-            if config.env.lower() not in WorkspaceEnv.values_list():
-                logger.error(f"{config.env} not supported")
-                continue
-            config_env = WorkspaceEnv.from_str(config.env)
-
-            if config_env == target_env:
-                configs_to_shutdown_env_filter.append(config)
-    # logger.debug("Shutting down configs: {}".format(configs_to_shutdown_env_filter))
-
-    num_configs_to_process = len(configs_to_shutdown_env_filter)
-    num_configs_processed = 0
-    for config in configs_to_shutdown_env_filter:
-        if not isinstance(config, InfraConfig):
-            logger.error(f"{config} is not an instance of InfraConfig")
-            continue
-        if not config.is_valid():
-            logger.error("Config invalid")
-            continue
-        if not config.enabled:
-            logger.debug(
-                f"{config.__class__.__name__} for env {config.__class__.env} disabled"
-            )
-            num_configs_processed += 1
-            continue
-
-        ######################################################################
-        # NOTE: VERY IMPORTANT TO GET RIGHT
-        # Update sub-configs data using WorkspaceConfig
-        # 1. Pass down the paths from the WorkspaceConfig
-        #       These paths are used everywhere from Infra to Apps
-        # 2. Pass down common cloud configuration. eg: aws_region, aws_profile
-        ######################################################################
-
-        # The ws_root_path is the ROOT directory for the workspace
-        config.workspace_root_path = ws_data.ws_root_path
-        config.workspace_config_file_path = ws_data.ws_config_file_path
-        config.scripts_dir = ws_config.scripts_dir
-        config.storage_dir = ws_config.storage_dir
-        config.meta_dir = ws_config.meta_dir
-        config.products_dir = ws_config.products_dir
-        config.notebooks_dir = ws_config.notebooks_dir
-        config.workspace_config_dir = ws_config.workspace_config_dir
+    configs_to_shutdown: List[InfraConfig] = filter_and_prep_configs(
+        ws_data=ws_data,
+        target_env=target_env,
+        target_config=target_config,
+        order="delete",
+    )
 
+    num_configs_to_shutdown = len(configs_to_shutdown)
+    num_configs_shutdown = 0
+    for config in configs_to_shutdown:
         logger.debug(f"Shutting down {config.__class__.__name__}")
         if isinstance(config, DockerConfig):
             from phiterm.docker.docker_operator import shutdown_docker_config
 
-            # Pass down docker_env from WorkspaceConfig
-            config.docker_env = ws_config.docker_env
             shutdown_docker_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_shutdown += 1
         if isinstance(config, K8sConfig):
             from phiterm.k8s.k8s_operator import shutdown_k8s_config
 
-            # Pass down k8s_env from WorkspaceConfig
-            config.k8s_env = ws_config.k8s_env
             shutdown_k8s_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_shutdown += 1
         if isinstance(config, AwsConfig):
             from phiterm.aws.aws_operator import shutdown_aws_config
 
-            # Pass down common aws configuration from WorkspaceConfig
-            config.aws_region = ws_config.aws_region
-            config.aws_profile = ws_config.aws_profile
-            config.aws_config_file = ws_config.aws_config_file
-            config.aws_shared_credentials_file = ws_config.aws_shared_credentials_file
             shutdown_aws_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_shutdown += 1
+        # white space between runs
+        print_info("")
 
     print_info(
-        f"\n# Configs processed: {num_configs_processed}/{num_configs_to_process}"
+        f"\n# Configs shutdown: {num_configs_shutdown}/{num_configs_to_shutdown}"
     )
-    if num_configs_to_process == num_configs_processed:
+    if num_configs_to_shutdown == num_configs_shutdown:
         if not dry_run:
             print_subheading("Workspace shutdown success")
     else:
@@ -823,7 +810,9 @@ def patch_workspace(
     target_config: Optional[WorkspaceConfigType] = None,
     target_name: Optional[str] = None,
     target_type: Optional[str] = None,
+    target_app: Optional[str] = None,
     dry_run: Optional[bool] = False,
+    auto_confirm: Optional[bool] = False,
 ) -> None:
     """Patch the Phidata Workspace. This is called from `phi ws patch`"""
 
@@ -831,128 +820,58 @@ def patch_workspace(
         logger.error("WorkspaceConfig invalid")
         return
 
-    ws_config: WorkspaceConfig = ws_data.ws_config
-
-    # Add configs in the order they should be patched
-    configs_to_patch_infra_filter: List[InfraConfig] = []
-    if target_config is None:
-        if ws_config.docker is not None:
-            configs_to_patch_infra_filter.extend(ws_config.docker)
-        if ws_config.k8s is not None:
-            configs_to_patch_infra_filter.extend(ws_config.k8s)
-        if ws_config.aws is not None:
-            configs_to_patch_infra_filter.extend(ws_config.aws)
-    elif target_config == WorkspaceConfigType.docker:
-        if ws_config.docker is not None:
-            configs_to_patch_infra_filter.extend(ws_config.docker)
-        else:
-            logger.error("No DockerConfig provided")
-    elif target_config == WorkspaceConfigType.k8s:
-        if ws_config.k8s is not None:
-            configs_to_patch_infra_filter.extend(ws_config.k8s)
-        else:
-            logger.error("No K8sConfig provided")
-    elif target_config == WorkspaceConfigType.aws:
-        if ws_config.aws is not None:
-            configs_to_patch_infra_filter.extend(ws_config.aws)
-        else:
-            logger.error("No AwsConfig provided")
-
-    configs_to_patch_env_filter: List[InfraConfig] = []
-    if target_env is None:
-        configs_to_patch_env_filter = configs_to_patch_infra_filter
-    else:
-        for config in configs_to_patch_infra_filter:
-            if config.env is None:
-                continue
-            if config.env.lower() not in WorkspaceEnv.values_list():
-                logger.error(f"{config.env} not supported")
-                continue
-            config_env = WorkspaceEnv.from_str(config.env)
-
-            if config_env == target_env:
-                configs_to_patch_env_filter.append(config)
-    # logger.debug("Patching configs: {}".format(configs_to_shutdown_env_filter))
-
-    num_configs_to_process = len(configs_to_patch_env_filter)
-    num_configs_processed = 0
-    for config in configs_to_patch_env_filter:
-        if not isinstance(config, InfraConfig):
-            logger.error(f"{config} is not an instance of InfraConfig")
-            continue
-        if not config.is_valid():
-            logger.error("Config invalid")
-            continue
-        if not config.enabled:
-            logger.debug(
-                f"{config.__class__.__name__} for env {config.__class__.env} disabled"
-            )
-            num_configs_processed += 1
-            continue
-
-        ######################################################################
-        # NOTE: VERY IMPORTANT TO GET RIGHT
-        # Update sub-configs data using WorkspaceConfig
-        # 1. Pass down the paths from the WorkspaceConfig
-        #       These paths are used everywhere from Infra to Apps
-        # 2. Pass down common cloud configuration. eg: aws_region, aws_profile
-        ######################################################################
-
-        # The ws_root_path is the ROOT directory for the workspace
-        config.workspace_root_path = ws_data.ws_root_path
-        config.workspace_config_file_path = ws_data.ws_config_file_path
-        config.scripts_dir = ws_config.scripts_dir
-        config.storage_dir = ws_config.storage_dir
-        config.meta_dir = ws_config.meta_dir
-        config.products_dir = ws_config.products_dir
-        config.notebooks_dir = ws_config.notebooks_dir
-        config.workspace_config_dir = ws_config.workspace_config_dir
+    configs_to_patch: List[InfraConfig] = filter_and_prep_configs(
+        ws_data=ws_data,
+        target_env=target_env,
+        target_config=target_config,
+        order="create",
+    )
 
+    num_configs_to_patch = len(configs_to_patch)
+    num_configs_patched = 0
+    for config in configs_to_patch:
         logger.debug(f"Patching {config.__class__.__name__}")
         if isinstance(config, DockerConfig):
             from phiterm.docker.docker_operator import patch_docker_config
 
-            # Pass down docker_env from WorkspaceConfig
-            config.docker_env = ws_config.docker_env
             patch_docker_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_patched += 1
         if isinstance(config, K8sConfig):
             from phiterm.k8s.k8s_operator import patch_k8s_config
 
-            # Pass down k8s_env from WorkspaceConfig
-            config.k8s_env = ws_config.k8s_env
             patch_k8s_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_patched += 1
         if isinstance(config, AwsConfig):
             from phiterm.aws.aws_operator import patch_aws_config
 
-            # Pass down common aws configuration from WorkspaceConfig
-            config.aws_region = ws_config.aws_region
-            config.aws_profile = ws_config.aws_profile
-            config.aws_config_file = ws_config.aws_config_file
-            config.aws_shared_credentials_file = ws_config.aws_shared_credentials_file
             patch_aws_config(
                 config=config,
                 name_filter=target_name,
                 type_filter=target_type,
+                app_filter=target_app,
                 dry_run=dry_run,
+                auto_confirm=auto_confirm,
             )
-            num_configs_processed += 1
+            num_configs_patched += 1
+        # white space between runs
+        print_info("")
 
-    print_info(
-        f"\n# Configs processed: {num_configs_processed}/{num_configs_to_process}"
-    )
-    if num_configs_to_process == num_configs_processed:
+    print_info(f"\n# Configs patched: {num_configs_patched}/{num_configs_to_patch}")
+    if num_configs_to_patch == num_configs_patched:
         if not dry_run:
             print_subheading("Workspace patch success")
     else:
